#include <libgen.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/prctl.h>
#include <unistd.h>

#include "tal_video_enc.h"
#include "tuya_iot_config.h"
#include "tuya_ipc_api.h"
#include "tuya_ipc_media.h"
#include "tuya_ipc_media_stream_event.h"
#include "tuya_ipc_p2p.h"
#include "tuya_ipc_stream_storage.h"
#include "tuya_ring_buffer.h"
#include "ty_sdk_common.h"
#include "utilities/uni_log.h"
#include "tuya_p2p_api.h"

#define MAX_SNAPSHOT_BUFFER_SIZE_KB (100) // in KB

#define AUDIO_FRAME_SIZE 640
#define AUDIO_FPS 25
#define VIDEO_BUF_SIZE (1024 * 400) // Maximum frame

typedef struct
{
    BOOL_T enabled;
    TRANSFER_VIDEO_CLARITY_TYPE_E live_clarity;
    UINT_T max_users;
    TUYA_CODEC_ID_E p2p_audio_codec;
} TUYA_APP_P2P_MGR;

STATIC TUYA_APP_P2P_MGR s_p2p_mgr = { 0 };

STATIC VOID __TUYA_APP_media_frame_TO_trans_video(IN CONST MEDIA_FRAME_T* p_in, INOUT MEDIA_VIDEO_FRAME_T* p_out)
{
    UINT_T codec_type = 0;
    codec_type = (p_in->type & 0xff00) >> 8;
    p_out->video_codec = (codec_type == 0 ? TUYA_CODEC_VIDEO_H264 : TUYA_CODEC_VIDEO_H265);
    p_out->video_frame_type = (p_in->type && 0xff) == E_VIDEO_PB_FRAME ? TUYA_VIDEO_FRAME_PBFRAME : TUYA_VIDEO_FRAME_IFRAME;
    p_out->p_video_buf = p_in->p_buf;
    p_out->buf_len = p_in->size;
    p_out->pts = p_in->pts;
    p_out->timestamp = p_in->timestamp;

    return;
}

STATIC VOID __TUYA_APP_media_frame_TO_trans_audio(IN CONST MEDIA_FRAME_T* p_in, INOUT MEDIA_AUDIO_FRAME_T* p_out)
{
    DEVICE_MEDIA_INFO_T media_info = { 0 };
    tuya_ipc_media_adapter_get_media_info(0, 0, &media_info);
    p_out->audio_codec = media_info.av_encode_info.audio_codec[E_IPC_STREAM_AUDIO_MAIN];
    p_out->audio_sample = media_info.av_encode_info.audio_sample[E_IPC_STREAM_AUDIO_MAIN];
    p_out->audio_databits = media_info.av_encode_info.audio_databits[E_IPC_STREAM_AUDIO_MAIN];
    p_out->audio_channel = media_info.av_encode_info.audio_channel[E_IPC_STREAM_AUDIO_MAIN];
    p_out->p_audio_buf = p_in->p_buf;
    p_out->buf_len = p_in->size;
    p_out->pts = p_in->pts;
    p_out->timestamp = p_in->timestamp;

    return;
}
STATIC INT_T TUYA_APP_Enable_Speaker_CB(IN BOOL_T enable)
{
    return 0;
}

STATIC VOID __TUYA_APP_ss_pb_event_cb(IN CONST CHAR_T *sub_node_id, IN CONST UINT_T client_idx, IN SS_PB_EVENT_E pb_event)
{
    PR_DEBUG("ss pb rev event: %u %d", client_idx, pb_event);
    if (pb_event == SS_PB_FINISH) {
        tuya_media_playback_send_finish(sub_node_id, client_idx);
    }
}

STATIC VOID __TUYA_APP_ss_pb_get_video_cb(CONST CHAR_T *sub_node_id, IN CONST UINT_T client_idx, IN CONST MEDIA_FRAME_T* p_frame)
{
    MEDIA_VIDEO_FRAME_T video_frame;
    memset(&video_frame, 0, sizeof(video_frame));
    __TUYA_APP_media_frame_TO_trans_video(p_frame, &video_frame);

    PR_ERR("send pb V typ:%d,sz:%u,tm:%llu", video_frame.video_frame_type, video_frame.buf_len, video_frame.timestamp);
    tuya_media_playback_send_video_frame(sub_node_id, client_idx, &video_frame);
}

STATIC VOID __TUYA_APP_ss_pb_get_audio_cb(CONST CHAR_T *sub_node_id, IN CONST UINT_T client_idx, IN CONST MEDIA_FRAME_T* p_frame)
{
    MEDIA_AUDIO_FRAME_T audio_frame;
    memset(&audio_frame, 0, sizeof(audio_frame));
    __TUYA_APP_media_frame_TO_trans_audio(p_frame, &audio_frame);

    tuya_media_playback_send_audio_frame(sub_node_id, client_idx, &audio_frame);
}

/* Callback functions for transporting events */
INT_T TUYA_IPC_p2p_event_cb(IN CONST INT_T device, IN CONST INT_T channel, IN CONST MEDIA_STREAM_EVENT_E event, IN PVOID_T args)
{
    int ret = 0;
    PR_DEBUG("p2p rev event cb=[%d] ", event);

    switch (event) {
    case MEDIA_STREAM_VIDEO_START_GW: {
        C2C_TRANS_CTRL_GW_VIDEO_START* parm = (C2C_TRANS_CTRL_GW_VIDEO_START*)args;
        PR_DEBUG("device[%u] sub_id[%s] video start", parm->device, parm->sub_id);
        break;
    }
    case MEDIA_STREAM_VIDEO_STOP_GW: {
        C2C_TRANS_CTRL_GW_VIDEO_STOP* parm = (C2C_TRANS_CTRL_GW_VIDEO_STOP*)args;
        PR_DEBUG("device[%u] sub_id[%s] video stop", parm->device, parm->sub_id);
        break;
    }
    case MEDIA_STREAM_AUDIO_START_GW: {
        C2C_TRANS_CTRL_GW_AUDIO_START* parm = (C2C_TRANS_CTRL_GW_AUDIO_START*)args;
        PR_DEBUG("device[%u] sub_id[%s] audio start", parm->device, parm->sub_id);
        break;
    }
    case MEDIA_STREAM_AUDIO_STOP_GW: {
        C2C_TRANS_CTRL_GW_AUDIO_STOP* parm = (C2C_TRANS_CTRL_GW_AUDIO_STOP*)args;
        PR_DEBUG("device[%u] sub_id[%s] audio stop", parm->device, parm->sub_id);
        break;
    }
    case MEDIA_STREAM_SPEAKER_START_GW: {
        PR_DEBUG("enable audio speaker");
        break;
    }
    case MEDIA_STREAM_SPEAKER_STOP_GW: {
        PR_DEBUG("disable audio speaker");
        break;
    }
    case MEDIA_STREAM_VIDEO_CLARITY_SET_GW: {
        C2C_TRANS_LIVE_GW_CLARITY_PARAM_S* parm = (C2C_TRANS_LIVE_GW_CLARITY_PARAM_S *)args;
        PR_DEBUG("set device[%d] clarity:%d", parm->device, parm->clarity);
        break;
    }
    case MEDIA_STREAM_PLAYBACK_QUERY_MONTH_SIMPLIFY_GW: {
        C2C_TRANS_QUERY_GW_PB_MONTH_REQ* p = (C2C_TRANS_QUERY_GW_PB_MONTH_REQ*)args;
        PR_DEBUG("pb query by month:[%s]idx[%d],%d-%d", p->sub_id, p->idx, p->year, p->month);

        OPERATE_RET ret = tuya_ipc_ss_pb_query_by_month(p->sub_id, p->idx, p->year, p->month, &(p->day));
        if (OPRT_OK != ret) {
            PR_ERR("pb query by month: %d-%d ret:%d", p->year, p->month, ret);
        }

        break;
    }
    case MEDIA_STREAM_PLAYBACK_QUERY_DAY_TS_GW: {
        C2C_TRANS_QUERY_GW_PB_DAY_RESP* pquery = (C2C_TRANS_QUERY_GW_PB_DAY_RESP*)args;
        PR_DEBUG("pb_ts query by day:[%s]idx[%d]%d-%d-%d", pquery->sub_id, pquery->idx, pquery->year, pquery->month, pquery->day);
        SS_QUERY_DAY_TS_ARR_T* p_day_ts = NULL;
        OPERATE_RET ret = tuya_ipc_ss_pb_query_by_day(pquery->sub_id, pquery->idx, 
                                                        pquery->year, pquery->month, pquery->day, &p_day_ts);
        if (OPRT_OK != ret) {
            PR_ERR("pb_ts query by day: idx[%d] %d-%d-%d Fail", pquery->idx, pquery->year, pquery->month, pquery->day);
            break;
        }
        if (p_day_ts) {
            PR_INFO("%s %d count = %d\n", __FUNCTION__, __LINE__, p_day_ts->file_count);
            UINT32_T result_sz = sizeof(PLAY_BACK_ALARM_INFO_ARR) + p_day_ts->file_count * sizeof(PLAY_BACK_ALARM_FRAGMENT);
            PLAY_BACK_ALARM_INFO_ARR* pResult = (PLAY_BACK_ALARM_INFO_ARR*)malloc(result_sz);
            if (NULL == pResult) {
                PR_ERR("%s %d malloc failed \n", __FUNCTION__, __LINE__);
                tuya_ipc_ss_pb_query_free_ts_arr(p_day_ts);
                pquery->alarm_arr = NULL;
                break;
            }
            memset(pResult, 0, result_sz);

            INT_T i;
            pResult->file_count = p_day_ts->file_count;
            for (i = 0; i < p_day_ts->file_count; i++) {
                pResult->file_arr[i].type = p_day_ts->file_arr[i].type;
                pResult->file_arr[i].video_type = p_day_ts->file_arr[i].video_type;
                pResult->file_arr[i].time_sect.start_timestamp = p_day_ts->file_arr[i].start_timestamp;
                pResult->file_arr[i].time_sect.end_timestamp = p_day_ts->file_arr[i].end_timestamp;
                PR_DEBUG("==>QUERY_DAY:%d,%d,%u,%u", pResult->file_arr[i].type, pResult->file_arr[i].video_type, 
                                                pResult->file_arr[i].time_sect.start_timestamp, pResult->file_arr[i].time_sect.end_timestamp);
            }
            pquery->alarm_arr = pResult;
            tuya_ipc_ss_pb_query_free_ts_arr(p_day_ts);
        } else {
            pquery->alarm_arr = NULL;
        }
        break;
    }
    case MEDIA_STREAM_PLAYBACK_START_TS_GW: {
        C2C_TRANS_CTRL_GW_PB_START* p_start = (C2C_TRANS_CTRL_GW_PB_START*)args;
        PR_DEBUG("PB StartTS dev:[%s]idx[%d]%u[%u %u]", p_start->sub_id, p_start->idx, p_start->playTime,
                                p_start->time_sect.start_timestamp, p_start->time_sect.end_timestamp);

        SS_FILE_TIME_TS_T pb_file_info;
        memset(&pb_file_info, 0x00, sizeof(SS_FILE_TIME_TS_T));
        
        pb_file_info.start_timestamp = p_start->time_sect.start_timestamp;
        pb_file_info.end_timestamp = p_start->time_sect.end_timestamp;
        int ret = tuya_ipc_ss_pb_start(p_start->sub_id, p_start->idx, 0, 
                                        __TUYA_APP_ss_pb_event_cb,
                                        __TUYA_APP_ss_pb_get_video_cb,
                                        __TUYA_APP_ss_pb_get_audio_cb);
        if (0 != ret) {
            PR_ERR("pb_start failed");
            tuya_imm_p2p_playback_send_finish(p_start->sub_id, p_start->idx);
        } else {
            if (0 != tuya_ipc_ss_pb_seek(p_start->sub_id, p_start->idx, 0, &pb_file_info, p_start->playTime)) {
                PR_ERR("pb_seek failed");
                tuya_imm_p2p_playback_send_finish(p_start->sub_id, p_start->idx);
            }
        }

        break;
    }
    case MEDIA_STREAM_PLAYBACK_PAUSE_GW: {
        C2C_TRANS_CTRL_GW_PB_PAUSE* p_pause = (C2C_TRANS_CTRL_GW_PB_PAUSE*)args;
        PR_DEBUG("PB Pause dev:[%s]idx[%d]", p_pause->sub_id, p_pause->idx);

        tuya_ipc_ss_pb_set_status(p_pause->sub_id, p_pause->idx, 0, SS_PB_PAUSE);
        break;
    }
    case MEDIA_STREAM_PLAYBACK_RESUME_GW: {
        C2C_TRANS_CTRL_GW_PB_RESUME* p_resume = (C2C_TRANS_CTRL_GW_PB_RESUME*)args;
        PR_DEBUG("PB Resume dev:[%s]idx[%d]", p_resume->sub_id, p_resume->idx);

        tuya_ipc_ss_pb_set_status(p_resume->sub_id, p_resume->idx, 0, SS_PB_RESUME);
        break;
    }
    case MEDIA_STREAM_PLAYBACK_MUTE_GW: {
        C2C_TRANS_CTRL_GW_PB_MUTE* p_mute = (C2C_TRANS_CTRL_GW_PB_MUTE*)args;
        PR_DEBUG("PB MUTE dev:[%s]idx[%d]", p_mute->sub_id, p_mute->idx);

        tuya_ipc_ss_pb_set_status(p_mute->sub_id, p_mute->idx, 0, SS_PB_MUTE);
        break;
    }
    case MEDIA_STREAM_PLAYBACK_UNMUTE_GW: {
        C2C_TRANS_CTRL_GW_PB_UNMUTE* p_unmute = (C2C_TRANS_CTRL_GW_PB_UNMUTE*)args;
        PR_DEBUG("PB unmute dev:[%s]idx[%d]", p_unmute->sub_id, p_unmute->idx);

        tuya_ipc_ss_pb_set_status(p_unmute->sub_id, p_unmute->idx, 0, SS_PB_UN_MUTE);
        break;
    }
    case MEDIA_STREAM_PLAYBACK_SPEED_GW: {
        C2C_TRANS_CTRL_PB_SET_SPEED_GW* p_speed = (C2C_TRANS_CTRL_PB_SET_SPEED_GW*)args;
        PR_DEBUG("PB SPEED dev:[%s]idx[%d] set speed[%u]", p_speed->sub_id, p_speed->idx, p_speed->speed);
        int ret = tuya_ipc_ss_pb_set_speed(p_speed->sub_id, p_speed->idx, 0, p_speed->speed);

        if (0 != ret) {
            PR_ERR("pb set speed failed");
            tuya_imm_p2p_playback_send_finish(p_speed->sub_id, p_speed->idx);
        }
        break;
    }
    case MEDIA_STREAM_PLAYBACK_STOP_GW: {
        C2C_TRANS_CTRL_GW_PB_STOP* p_stop = (C2C_TRANS_CTRL_GW_PB_STOP*)args;
        PR_DEBUG("PB Stop dev:[%s]idx[%d]", p_stop->sub_id, p_stop->idx);

        tuya_ipc_ss_pb_stop(p_stop->sub_id, p_stop->idx);
        break;
    }
    case MEDIA_STREAM_PLAYBACK_DELETE_GW: {
        C2C_TRANS_CTRL_PB_DELDATA_BYDAY_GW_REQ *p_deldata_byday = (C2C_TRANS_CTRL_PB_DELDATA_BYDAY_GW_REQ *)args;
        PR_DEBUG("PB deldata_byday idx[%d],%u-%u-%u", p_deldata_byday->idx, p_deldata_byday->year, 
                                                    p_deldata_byday->month, p_deldata_byday->day);

        if (OPRT_OK != tuya_ipc_ss_delete_video(NULL, p_deldata_byday->idx, 
                                            p_deldata_byday->year, p_deldata_byday->month, p_deldata_byday->day)) {
            PR_ERR("tuya_ipc_ss_delete_video err");
        }
        break;
    }
    case MEDIA_STREAM_DOWNLOAD_START_GW: {
        C2C_TRANS_CTRL_GW_DL_START* p_dl_start = (C2C_TRANS_CTRL_GW_DL_START*)args;
        PR_DEBUG("PB DL dev:[%s]idx[%d] [%u,%u]", p_dl_start->sub_id, p_dl_start->idx, 
                                            p_dl_start->downloadStartTime, p_dl_start->downloadEndTime);

        SS_DOWNLOAD_FILES_TS_T strParm = { 0 };
        strParm.dl_start_time = p_dl_start->downloadStartTime;
        strParm.dl_end_time = p_dl_start->downloadEndTime;
        strParm.file_count = 0;             // TODO
        strParm.p_file_info_arr = NULL;     // TODO

        if (OPRT_OK == tuya_ipc_ss_donwload_pre(p_dl_start->sub_id, p_dl_start->idx, &strParm)) {
            tuya_ipc_ss_download_set_status(p_dl_start->sub_id, p_dl_start->idx, SS_DL_START);
        }
        if (strParm.p_file_info_arr) {
            free(strParm.p_file_info_arr);
        }
        break;
    }
    case MEDIA_STREAM_DOWNLOAD_PAUSE_GW: {
        C2C_TRANS_CTRL_GW_DL_PAUSE* p_pause = (C2C_TRANS_CTRL_GW_DL_PAUSE*)args;
        PR_DEBUG("PB DL PAUSE dev:[%s]idx[%d]", p_pause->sub_id, p_pause->idx);

        tuya_ipc_ss_download_set_status(p_pause->sub_id, p_pause->idx, SS_DL_PAUSE);

        break;
    }
    case MEDIA_STREAM_DOWNLOAD_RESUME_GW: {
        C2C_TRANS_CTRL_GW_DL_RESUME* p_resume = (C2C_TRANS_CTRL_GW_DL_RESUME*)args;
        PR_DEBUG("PB DL resume dev:[%s]idx[%d]", p_resume->sub_id, p_resume->idx);

        tuya_ipc_ss_download_set_status(p_resume->sub_id, p_resume->idx, SS_DL_RESUME);
        break;
    }
    case MEDIA_STREAM_DOWNLOAD_CANCLE_GW: {
        C2C_TRANS_CTRL_GW_DL_CANCLE* p_cancel = (C2C_TRANS_CTRL_GW_DL_CANCLE*)args;
        PR_DEBUG("PB DL cancel dev:[%s]idx[%d]", p_cancel->sub_id, p_cancel->idx);
        tuya_ipc_ss_download_set_status(p_cancel->sub_id, p_cancel->idx, SS_DL_CANCEL);
        break;
    }
    default:
        PR_ERR("not supprot cmd:%d", event);
        break;
    }
    return ret;
}

VOID TUYA_IPC_APP_rev_audio_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_AUDIO_FRAME_T* p_audio_frame)
{
    MEDIA_FRAME_T audio_frame = { 0 };
    audio_frame.p_buf = p_audio_frame->p_audio_buf;
    audio_frame.size = p_audio_frame->buf_len;

    PR_INFO("Rev Audio. size:[%u] audio_codec:[%d] audio_sample:[%d] audio_databits:[%d] audio_channel:[%d]\n", p_audio_frame->buf_len,
            p_audio_frame->audio_codec, p_audio_frame->audio_sample, p_audio_frame->audio_databits, p_audio_frame->audio_channel);

    //    TUYA_APP_Rev_Audio_CB( &audio_frame, TUYA_AUDIO_SAMPLE_8K, TUYA_AUDIO_DATABITS_16, TUYA_AUDIO_CHANNEL_MONO);

    return;
}

VOID TUYA_IPC_APP_rev_video_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_VIDEO_FRAME_T* p_video_frame)
{
    PR_INFO("Rev video. size:[%u] video_codec:[%d] video_frame_type:[%d]\n", p_video_frame->buf_len, p_video_frame->video_codec, p_video_frame->video_frame_type);
    return;
}

VOID TUYA_IPC_APP_rev_file_cb(IN INT_T device, IN INT_T channel, IN CONST MEDIA_FILE_DATA_T* p_file_data)
{
    return;
}

VOID TUYA_APP_get_snapshot_cb(IN INT_T device, IN INT_T channel, OUT CHAR_T* snap_addr, OUT INT_T* snap_size)
{
    // in this demo ignore device and channel parameter
    IPC_APP_get_snapshot(snap_addr, snap_size);
}

VOID TUYA_IPC_Media_Adapter_Init(TUYA_IPC_SDK_MEDIA_ADAPTER_S* p_media_adatper_info, TUYA_IPC_SDK_MEDIA_STREAM_S* p_media_infos)
{
    TUYA_IPC_MEDIA_ADAPTER_VAR_T media_var;
    media_var.get_snapshot_cb = p_media_adatper_info->get_snapshot_cb;
    media_var.on_recv_audio_cb = p_media_adatper_info->rev_audio_cb;
    media_var.on_recv_video_cb = p_media_adatper_info->rev_video_cb;
    media_var.on_recv_file_cb = p_media_adatper_info->rev_file_cb;
    media_var.available_media_memory = 0;

    tuya_ipc_media_adapter_init(&media_var);

    DEVICE_MEDIA_INFO_T device_media_info = { 0 };
    memcpy(&device_media_info.av_encode_info, &p_media_infos->media_info, sizeof(IPC_MEDIA_INFO_T));

    device_media_info.audio_decode_info.enable = 1;
    device_media_info.audio_decode_info.audio_codec = TUYA_CODEC_AUDIO_G711U;
    device_media_info.audio_decode_info.audio_sample = TUYA_AUDIO_SAMPLE_8K;
    device_media_info.audio_decode_info.audio_databits = TUYA_AUDIO_DATABITS_16;
    device_media_info.audio_decode_info.audio_channel = TUYA_AUDIO_CHANNEL_MONO;

    device_media_info.max_pic_len = MAX_SNAPSHOT_BUFFER_SIZE_KB;

#ifdef VIDEO_DECODE //上报视频解码能力，用于双向视频功能
    TUYA_DECODER_T video_decoder = { 0 };
    video_decoder.codec_id = TUYA_CODEC_VIDEO_H264;
    video_decoder.decoder_desc.v_decoder.height = 320;
    video_decoder.decoder_desc.v_decoder.width = 240;
    video_decoder.decoder_desc.v_decoder.profile = VIDEO_AVC_PROFILE_BASE_LINE;
    device_media_info.decoder_cnt = 1;
    device_media_info.decoders = &video_decoder;
#endif
    tuya_ipc_media_adapter_set_media_info(0, 0, device_media_info);

    return;
}

VOID TUYA_IPC_Media_Stream_Init(TUYA_IPC_SDK_MEDIA_ADAPTER_S* p_media_adatper_info)
{
    MEDIA_STREAM_VAR_T media_stream_var = { 0 };
    media_stream_var.on_event_cb = p_media_adatper_info->media_event_cb;
    media_stream_var.max_client_num = p_media_adatper_info->max_stream_client;
    media_stream_var.def_live_mode = p_media_adatper_info->live_mode;
#ifdef VIDEO_DECODE //增加接收缓存区大小，用于双向视频功能
    media_stream_var.recv_buffer_size = 300 * 1024;
#else
    media_stream_var.recv_buffer_size = 16 * 1024;
#endif
    media_stream_var.lan_only_mode = tuya_get_lan_mode();
    int ret = tuya_ipc_media_stream_init(&media_stream_var);
    PR_DEBUG("media stream init result is %d\n", ret);

    return;
}

/*
---------------------------------------------------------------------------------
code related RingBuffer
---------------------------------------------------------------------------------
*/
IPC_MEDIA_INFO_T old_mediaInfo = { 0 };
RING_BUFFER_USER_HANDLE_T g_a_handle[MAX_DEV_NUM] = { NULL };
RING_BUFFER_USER_HANDLE_T g_v_handle[MAX_DEV_NUM] = { NULL };
RING_BUFFER_USER_HANDLE_T g_v_handle_sub[MAX_DEV_NUM] = { NULL };
STATIC BOOL_T s_ring_buffer_inited[MAX_DEV_NUM] = { FALSE };

OPERATE_RET TUYA_APP_Init_Ring_Buffer(CONST IPC_MEDIA_INFO_T* pMediaInfo, INT_T dev_index)
{
    if (NULL == pMediaInfo) {
        PR_ERR("create ring buffer para is NULL\n");
        return -1;
    }

    if (dev_index >= MAX_DEV_NUM) {
        PR_ERR("dev_index is err %d\n", dev_index);
        return -1;
    }

    OPERATE_RET ret = OPRT_OK;

    if (s_ring_buffer_inited[dev_index] == TRUE) {
        PR_DEBUG("The Ring Buffer Is Already Inited");
        return OPRT_OK;
    }

    IPC_STREAM_E ringbuffer_stream_type;
    // CHANNEL_E channel;
    RING_BUFFER_INIT_PARAM_T param = { 0 };

    for (ringbuffer_stream_type = E_IPC_STREAM_VIDEO_MAIN; ringbuffer_stream_type < E_IPC_STREAM_MAX; ringbuffer_stream_type++) {
        PR_DEBUG("init ring buffer dev_index:%d Stream:%d Enable:%d", dev_index, ringbuffer_stream_type, pMediaInfo->stream_enable[ringbuffer_stream_type]);
        if (pMediaInfo->stream_enable[ringbuffer_stream_type] == TRUE) {
            if (ringbuffer_stream_type == E_IPC_STREAM_AUDIO_MAIN) {
                param.bitrate = pMediaInfo->audio_sample[E_IPC_STREAM_AUDIO_MAIN] * pMediaInfo->audio_databits[E_IPC_STREAM_AUDIO_MAIN] / 1024;
                param.fps = pMediaInfo->audio_fps[E_IPC_STREAM_AUDIO_MAIN];
                param.max_buffer_seconds = 0;
                param.request_key_frame_cb = NULL;
                PR_DEBUG("audio_sample %d, audio_databits %d, audio_fps %d", pMediaInfo->audio_sample[E_IPC_STREAM_AUDIO_MAIN], pMediaInfo->audio_databits[E_IPC_STREAM_AUDIO_MAIN], pMediaInfo->audio_fps[E_IPC_STREAM_AUDIO_MAIN]);
                ret = tuya_ipc_ring_buffer_init(dev_index, 0, ringbuffer_stream_type, &param);
            } else {
                param.bitrate = pMediaInfo->video_bitrate[ringbuffer_stream_type];
                param.fps = pMediaInfo->video_fps[ringbuffer_stream_type];
                param.max_buffer_seconds = 0;
                param.request_key_frame_cb = NULL;
                PR_DEBUG("video_bitrate %d, video_fps %d", pMediaInfo->video_bitrate[ringbuffer_stream_type], pMediaInfo->video_fps[ringbuffer_stream_type]);
                ret = tuya_ipc_ring_buffer_init(dev_index, 0, ringbuffer_stream_type, &param);
            }

            if (ret != 0) {
                PR_ERR("init ring buffer fails. %d %d", ringbuffer_stream_type, ret);
                return OPRT_MALLOC_FAILED;
            }
            PR_DEBUG("init ring buffer success. channel:%d", ringbuffer_stream_type);
        }
    }

    memcpy(&old_mediaInfo, pMediaInfo, sizeof(IPC_MEDIA_INFO_T));
    s_ring_buffer_inited[dev_index] = TRUE;

    return OPRT_OK;
}

OPERATE_RET TUYA_APP_Put_Frame(RING_BUFFER_USER_HANDLE_T handle, IN CONST MEDIA_FRAME_T* p_frame)
{
    PR_TRACE("Put Frame. type:%d size:%u pts:%llu ts:%llu",
             p_frame->type, p_frame->size, p_frame->pts, p_frame->timestamp);

    OPERATE_RET ret = tuya_ipc_ring_buffer_append_data(handle, p_frame->p_buf, p_frame->size, p_frame->type, p_frame->pts);

    if (ret != OPRT_OK) {
        PR_ERR("Put Frame Fail.%d  type:%d size:%u pts:%llu ts:%llu", ret,
               p_frame->type, p_frame->size, p_frame->pts, p_frame->timestamp);
    }
    return ret;
}

int IPC_APP_get_snapshot(char* snap_addr, int* snap_size)
{
    TAL_VENC_FRAME_T frame = { 0 };
    if (snap_addr == NULL || snap_size == NULL || *snap_size <= 0) {
        PR_ERR("parm is wrong\n");
        return OPRT_COM_ERROR;
    }

    frame.pbuf = snap_addr;
    frame.buf_size = *snap_size;
    OPERATE_RET ret = tal_venc_get_frame(0, 4, &frame);
    if (ret != OPRT_OK) {
        PR_ERR("get pic failed\n", ret);
        return OPRT_COM_ERROR;
    }
    *snap_size = frame.used_size;
    PR_INFO("get pic suc, size:%d", frame.used_size);

    return 0;
}
